;Sprite Drawing Routine (SDR): 16 (+) bit version, transparency
;by JL <JayEll64@aol.com>
;inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;	ix=pointer to mask data
;sprite data stored like:
;.db (# of bytes across),(# of rows down)
;.db (sprite data)
;
;mask data (transparency) stored as:
;.db (mask data: 1=draw; 0='or' with background)
;
;size= 93 bytes (+6 pushes/pops)
;If you want the sprite to be drawn in the graphics memory instead of the
; video memory, simply replace the destination address before running the sprite routine : 
;ld hl,VIDEO_MEM
;ld (DEST_ADDR),hl

SDR16TR:
 di	;disable the interrupts
 push iy
 ld a,63	;-->flip x coordinate around for FIND_PIXEL
 sub c		; / may be removed if you account for it in your program
 ld c,a		;/	(save 4 bytes!)
 push hl	;push pointer to sprite data to the stack
 call FIND_PIXEL	;make a FIND_PIXEL
 ld de,(DEST_ADDR)	;define offset
 add hl,de	;add offset to get absolute address to place sprite
 ex (sp),hl	;hl(where to put sprite)<->(sp)(pointer to sprite)
 ld e,(hl)	;e=width of sprite
 inc hl	;next byte
 ld d,(hl)	;d=height of sprite
 inc hl	;hl points to actual sprite now
 ex (sp),hl	;trade back
 pop iy	;iy=pointer to sprite

 add a,a	;->sets all bits to the right of the set bit in reg a
 dec a		;/ for example, if a=%00010000 before, a=%00011111 aft
 ld c,a	;save mask into reg c
 ld a,b	;a=x coor
 and %00000111	;find remainder of a/8 (x coor divided by 8)
 inc a	;increase to get range of 1-8
 ld b,a	;and save into reg b

SDR16TR_NewRow:
 push de	;save counters
 push hl	;save where to place sprite
SDR16TR_NewCol:
 ld d,b	;save rotate counter into d
 ld a,(ix)	;load transparency byte into a
 inc ix	;ready for next byte

 rlca	;rotate left...
SDR16TR_PrepMask:
 rrca	;then rotate right b times (if b=1, stays the same, else shifts)
 djnz SDR16_PrepMask	;repeat

 ld b,a	;save a into b
 and c	;mask off bits
 cpl	;complement to mask off correct bits
 and (hl)	;mask off bits
 ld (hl),a	;and save
 inc hl	;next byte over
 ld a,c	;get mask
 cpl	;complement mask
 and b	;mask off other "half" of transparency
 cpl	;and complement
 and (hl)	;mask off bits
 ld (hl),a	;and save again
 dec hl	;back to where before

 ld b,d	;get counter back
 ld a,(iy)	;load sprite byte into a
 inc iy	;ready for next byte

 rlca	;rotate left...
SDR16TR_PrepByte:
 rrca	;then rotate right...
 djnz SDR16_PrepByte	;b times

 ld b,a	;save a into b
 and c	;mask off "half"
 or (hl)	;or it with background
 ld (hl),a	;and half of the sprite
 inc hl	;next byte over
 ld a,c	;load mask into a
 cpl	;complement
 and b	;mask off other "half" of bits
 or (hl)	;or it with background
 ld (hl),a	;and load into video location

 ld b,d	;restore rotate counter and mask
 dec e	;decrease width
 jr nz,SDR16TR_NewCol	;if not done, repeat
 pop hl	;restore video mem pointer
 ld de,$10	;de=$10
 add hl,de	;move hl one row down
 pop de	;restore counter
 dec d	;decrease number of rows
 jr nz,SDR16TR_NewRow	;if not done, repeat
 pop iy
 ei	;enable the interrupts
 ret	;return to main program