;Sprite Drawing Routine (SDR): 16 (+) bit version, clipping and transparency!
;by Joa <Joamail123@aol.com>
;inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;	ix=pointer to mask data
;Outputs (if sprite or partial sprite drawn):
;	af=destroyed
;	bc=destroyed
;	de=destroyed
;	hl=destroyed
;	ix=destroyed
;	no other registers are changed (iy is pushed and poped)
;	(If you want to save these registers, push them *before* the call
;	 and pop them back *after* the call, not *in* the call itself!)
;
;sprite data stored like:
;
;.db (# of bytes across),(# of rows down)
;.db (sprite data)
;
;size=240 bytes (+6 pushes/pops)
;
;
;If all your sprites are the same height, you can delete the 4 lines next
; "SDR16CLT:"  (ld e,(hl)\inc hl\ld d,(hl)\inc hl) and store all sprites
; with only the sprite data, leaving off that both byte that defines the
; width and height (saves 4 bytes + 2*# of sprites!)
;If you want the sprite to be drawn in the graphics memory instead of the
; video memory, simply replace the destination address before running the sprite routine : 
;ld hl,VIDEO_MEM
;ld (DEST_ADDR),hl

;***THIS ROUTINE ASSUMES THAT THE ORIGIN (0,0) IS THE *BOTTOM* LEFT CORNER!
; YOU'LL HAVE TO CHANGE THE ROUTINE SLIGHTLY IF YOU WANT THE ORIGIN TO BE
; THE TOP LEFT CORNER (JUST THE CLIPPING PART, I THINK)!***
;
;***You also need to define a byte labeled "ClipFlag".***
;
;
SDR16CLT_Exit:	;it's put there because I can make a relative jump
 pop hl	;pop the pushed registers
 pop iy
 ei
 ret	;finished, the sprite is not shown

SDR16CLT:
 di
 push iy
 ld e,(hl)	;e=width of sprite
 inc hl
 ld d,(hl)	;d=height of sprite
 inc hl	;now point to actual sprite
 xor a
 ld (ClipFlag),a ;flag=0 (meaning non-clipping)

 ld a,c	;a=y coor
 inc a	;a=y coor+1
 cp d	;compare to number of rows to display
 jr nc,SDR16CLT_CheckTClip	;if no carry, no bottom clipping
 ld d,a	;else, load y coor+1 into row counter
SDR16CLT_CheckTClip:
 sub 65	;a=65-a (check for top clipping)
 jr c,SDR16CLT_CheckLClip	;if carry no top clipping
 inc a	;else, a=a+1
 push hl 	;save pointer to sprite
 ex de,hl	;de can be used
 ld d,a	;save into couter
 sub h	;subtract number of rows to put
 jr nc,SDR16CLT_Exit
 neg	;make a positive
 ld h,a	;and that's the number of rows to display
 ld c,63	;and here's the y coor we're going to start from
 ld a,l	;load temporary the number of col into a
 ex (sp),hl	;restore pointer to sprite into hl and push the size of the sprite
 push bc	;save the coor because we're going to use bc
 ld b,0	;clear b
 ld c,a	;and store number of col into c
SDR16CLT_PrepHL:
 add hl,bc	;point to the next row of the sprite
 add ix,bc	;point to the next row of the mask
 dec d	;decrease counter
 jr nz,SDR16CLT_PrepHL	;and repeat
 pop bc	;restore the coor 
 pop de	;restore the size of the sprite

SDR16CLT_CheckLClip:
 ld a,b	;a=x coor
 cp 128	;check if start of sprite drawing within screen (0-127)
 jr c,SDR16CLT_DoneClip	;if so, done clipping (right clipping is "automatic")
 push de ;save size
 ld a,e	;a=number of col
 add a,a
 add a,a	;mul a with 8 
 add a,a	;a= number of pixels to display
 neg 	;make it negative
 cp b	;check if start too far left
 jr nc,SDR16CLT_Exit	;if so, sprite won't show
 ld e,0	;clear e
 ld a,b	;a=x coor (which is negative)
SDR16CLT_HowManyLClip:
 inc e
 add a,8	;add x coor with 8
 jr nc,SDR16CLT_HowManyLClip	;check  if positive, if not repeat until positive
 ld a,e	;load number of left clipping into a (since we can't do "ld (ClipFlag),e"
 ld (ClipFlag),a	;if a=1, 1 left clipping. If e=2, 1byte to add with and one left clippig, ...
 pop de	;restore size
 dec a	;decrease number of left clipping
 sub e	;subtract with actual nuber of col
 neg	;make it positive
 ld e,a	;that's the number of col to display
 inc c	;increase y coor (move it up one)
 ld a,b	;
 add a,128	;add 128 to x coor to make it start on right side and wrap
 ld b,a	;store the new coor

SDR16CLT_DoneClip:
 push hl	;save pointer to sprite
 push de	;save size
 call FIND_PIXEL
 ld de,(DEST_ADDR)	;define offset
 add hl,de	;and add offset to get where to put the sprite

 add a,a	;set all bits to the right of the set bit (sprite mask)
 dec a	;/
 ld c,a	;save sprite mask into c
 ld a,b	;get x coor
 and %00000111	;find remainder of x/8
 inc a	;increase to get range of 1-8
 ld b,a	;b=rotate counter
 pop de	;restore size
 pop iy	;restore pointer to sprite into iy

SDR16CLT_NewRow:
 push de	;save size (again)
 ld a,(ClipFlag)
 push af	;save number of byte...
 push hl	;save where to place the sprite
 or a	;check if left clipping or not
 jr z,SDR16CLT_NoLeftClip	;if so skip the pointing to the last col of the row
 push bc	;save rotate counter and sprite mask
 dec a	;a=number of byte to add with
 ld c,a	;
 ld b,0	;load that number into bc
 add hl,bc	;hl=poited to the last col of row of the video_mem
 add iy,bc	;iy=pointed to the first byte to put
 add ix,bc	;ix=pointed to the first byte to mask
 ld a,1	;load 1 into ClipFlag 
 ld (ClipFlag),a	;which means there is just one left clipping
 pop bc
SDR16CLT_NoLeftClip:
 push ix	;save beginning position of part of mask
 push iy	;save beginning position of part of  sprite to put
SDR16CLT_NewCol:
 ld a,(ix)	;load the current byte of the mask
 inc ix	;next byte
 ld d,b

 rlca	;rotate left once...
SDR16CLT_PrepMask:
 rrca	;...then rotate right...
 djnz SDR16CLT_PrepMask	;...b times
 ld b,d
 ld d,a

 ld a,(iy)
 inc iy
 push bc

 rlca	;rotate left once...
SDR16CLT_PrepByte:
 rrca	;then rotate right...
 djnz SDR16CLT_PrepByte	;b times
 ld b,a	;save rotated sprite into b

 ld a,(ClipFlag)	;
 dec a	;check if left clipping or not
 jr z,SDR16CLT_SkipLeft	;if so...

 ld a,d
 and c
 cpl	;put left half of the mask
 and (hl)
 ld (hl),a

 ld a,b
 and c
 or (hl)	;put the left half of byte...
 ld (hl),a
 inc hl

 ld a,l	;a=lower byte of video location to put sprite
 and %00001111	;mask off upper nibble
 jr z,SDR16CLT_PopBC	;if zero, right clipping and go to a new row
 dec hl	;else, decrease hl to put it back
SDR16CLT_SkipLeft:
 xor a
 ld (ClipFlag),a	;load 0 into ClipFlag which means there won't be any other skip left
 inc hl	;the increase it for possible left clipping

 ld a,c
 cpl
 and d
 cpl
 and (hl)
 ld (hl),a

 ld a,c
 cpl
 and b
 or (hl)	;put the right half of the byte
 ld (hl),a
 
 pop bc	;get the rotate counter back
 dec e	;decrease width
 jr nz,SDR16CLT_NewCol	;if not done, repeat
SDR16CLT_NextRow:
 pop iy	;iy=pointed to the beginning of part of sprite to put
 pop ix	;ix=pointed to the beginning of part of mask
 pop hl	;hl=pointed to where to put the sprite
 ld de,$10	;number of byte in a row
 add hl,de	;move hl one row down
 pop af	;restore the original ClipFlag
 ld (ClipFlag),a	;load it into its place
 pop de	;restore size
 ld a,d	;save number of row into a
 ld d,0	;
 add ix,de	;point the mask to the beggining of the next row
 add iy,de	;point the sprite to the beggining of the next row
 ld d,a	;restore size
 dec d	;decrease number of row
 jr nz,SDR16CLT_NewRow	;if not done, repeat
 pop iy
 ei
 ret	;finished, the sprite is shown with clipping or not

SDR16CLT_PopBC:
 pop bc	;pop it or the calc crash
 jr SDR16CLT_NextRow	;put a new row