;Sprite Drawing Routine (SDR): 8 bit version, clipping and transparency
;by Joa
;<Joamail123@aol.com>
;
;Inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;	ix=pointer to mask data (transparency)
;Outputs (if sprite or partial sprite drawn):
;	af=destroyed
;	bc=destroyed
;	de=destroyed
;	hl=pointer to byte after sprite
;	ix=pointer to byte after mask
;	no other registers are changed
;	(If you want to save these registers, push them *before* the call
;	 and pop them back *after* the call, not *in* the call itself!)
;
;Sprite data should be stored as follows:
;.db (# rows/bytes/height of sprite)
;.db (bitmap of sprite)
;Mask data (transparency) should stored as:
;.db (mask data: 1=draw, 0='or' with screen)
;
;Size=about 130 bytes (+pushes/pops)
;
;Notes:
;If all your sprites are the same height, you can delete the next 2
; lines (ld e,(hl)\inc hl); change the 3rd line after that (ld c,e) to
; 'ld c,x', where x is the height of your sprites; and store all sprites
; with only the sprite data, leaving off that first byte that defines the
; height (saves 1 bytes + # of sprites!)
;If you want the sprite to be drawn in the graphics memory instead of the
; video memory, simply replace the destination address before running the sprite routine : 
;ld hl,VIDEO_MEM
;ld (DEST_ADDR),hl
;
;***THIS ROUTINE ASSUMES THAT THE ORIGIN (0,0) IS THE *BOTTOM* LEFT CORNER!
; YOU'LL HAVE TO CHANGE THE ROUTINE SLIGHTLY IF YOU WANT THE ORIGIN TO BE
; THE TOP LEFT CORNER (JUST THE CLIPPING PART, I THINK)!***
;
;***You also need to define a byte labeled "ClipFlag".***

SDR8CLTR:
 ld e,(hl)	;-> can be replaced with 'ld e,8' if all your sprites are
 inc hl		;/	8x8
 xor a	;a=0
 ld (ClipFlag),a	;flag=0 (meaning non-clipping)

 ld a,c	;a=y coor
 inc a	;a=y coor+1
 cp e	;compare to number of rows to display
 jr nc,SDR8CLTR_CheckTClip	;if no carry, no bottom clipping
 ld e,a	;else, load y coor+1 into row counter
SDR8CLTR_CheckTClip:
 sub 65	;a=65-a	(checks for top clipping)
 jr c,SDR8CLTR_CheckLClip	;if a carry, no top clipping
 inc a	;else, a=a+1
 ld d,a	;save into counter
 sub e	;subtract number of rows to put
 ret nc	;if no carry, sprite won't even show up
 neg	;make a positive
 ld e,a	;and that's the number of rows to display
 ld c,63	;and here's the y coor we're going to start from
SDR8CLTR_PrepHL:
 inc hl	;next byte down in sprite
 inc ix
 dec d	;decrease counter
 jr nz,SDR8CLTR_PrepHL	;and repeat

SDR8CLTR_CheckLClip:	;now check left clipping
 ld a,b	;a=x coor
 cp 128	;check if start of sprite drawing is within screen (0-127)
 jr c,SDR8CLTR_DoneClip ;if so, done clipping (right clipping is "automatic")
 cp -7	;check if starts too far left
 ret c	;if so, sprite won't show
 inc c	;increase y coor (move it up one)
 add a,128	;add 128 to x coor to make it start on right side and wrap
 ld b,a	;save into x coor
 ld (ClipFlag),a	;and make flag non-zero

SDR8CLTR_DoneClip:
 push hl	;push pointer to sprite to stack
 call FIND_PIXEL	;)
 ld c,e	;c=number of rows in sprite
 ld de,(DEST_ADDR)	;define offset
 add hl,de	;and add offset to get where to put sprite

 add a,a	;-> sets all bits to the right of the set bit (sprite mask)
 dec a		;/
 ld e,a	;save sprite mask into e
 ld a,b	;get x coor
 and %00000111	;find remainder of x/8
 inc a	;increase to get range of 1-8
 ld b,a	;b=rotate counter

SDR8CLTR_NewRow:
 ld d,b
 ld a,(ix)
 inc ix

 rlca	;rotate left once...
SDR8CLTR_PrepMask:
 rrca	;...then rotate right...
 djnz SDR8CLTR_PrepMask	;...b times

 ld b,d
 ld d,a
 ex (sp),hl	;hl(video location)<->(sp)(pointer to sprite)
 ld a,(hl)	;a=byte in sprite
 inc hl	;next byte
 ex (sp),hl	;hl<->(sp)
 push bc	;save rotate counter and row counter

 rlca	;rotate left once...
SDR8CLTR_PrepByte:
 rrca	;...then rotate right...
 djnz SDR8CLTR_PrepByte	;...b times

 ld b,a	;save rotated sprite byte into b
 ld a,(ClipFlag)	;a=left clipping flag
 or a	;check if not 0
 jr nz,SDR8CLTR_SkipLeft	;if left clipping, only display right half of sprite

 ld a,d
 and e
 cpl
 and (hl)
 ld (hl),a

 ld a,b	;else, load rotated sprite byte back into a
 and e	;mask off left bits
 or (hl)
 ld (hl),a
 inc hl

 ld a,l	;a=lower byte of video location to put sprite
 and %00001111	;mask off upper nibble
 jr z,SDR8CLTR_NextRow ;if zero, hl now points to a new row, so right clipping
 dec hl	;else, decrease hl to put it back
SDR8CLTR_SkipLeft:
 inc hl	;then increase it for possible left clipping

 ld a,e
 cpl
 and d
 cpl
 and (hl)
 ld (hl),a

 ld a,e
 cpl
 and b
 or (hl)
 ld (hl),a

SDR8CLTR_NextRow:
 ld bc,$10-1	;bc=number of bytes in one row-1
 add hl,bc	;add to point hl to next row
 pop bc	;restore rotate counter and row counter
 dec c	;decrease row counter
 jr nz,SDR8CLTR_NewRow	;and repeat
 pop hl	;restore pointer to sprite (now points to byte after sprite)
 ret	;return