;Sprite Drawing Routine (SDR): 8 Bit Transparency Version
;by JL
;<JayEll64@aol.com>
;
;Inputs:
;	b=x coordinate
;	c=y coordinate
;	hl=pointer to sprite data
;	ix=pointer to mask data (transparency)
;Outputs:
;	af=destroyed
;	bc=destroyed
;	de=destroyed
;	hl=pointer to byte after sprite
;	ix=pointer to byte after mask
;	no other registers are changed
;If you want to save those registers that are destroyed, push/pop them!
;
;Sprite data should be stored as follows:
;.db (# rows/bytes/height of sprite)
;.db (bitmap of sprite)
;Mask data (transparency) should stored as:
;.db (mask data: 1=draw, 0='or' with screen)
;
;Size=76 bytes (+pushes/pops)
;
;Notes:
;You can make this routine smaller by deleting the first 3 lines and
; accounting in your program the fact the the origin (0,0) is the *bottom*
; left corner (saves 4 bytes!).
;Also, if all your sprites are the same height, you can delete the next 2
; lines (ld e,(hl)\inc hl); change the 3rd line after that (ld c,e) to
; 'ld c,x', where x is the height of your sprites; and store all sprites
; with only the sprite data, leaving off that first byte that defines the
; height (saves 1 byte + # of sprites!)
;If you want the sprite to be drawn in the graphics memory instead of the
; video memory, simply replace the destination address before running the sprite routine : 
;ld hl,VIDEO_MEM
;ld (DEST_ADDR),hl

SDR8TR:
 ld a,63
 sub c
 ld c,a
 ld e,(hl)
 inc hl
 push hl
 call FIND_PIXEL
 ld c,e
 ld de,VIDEO_MEM
 add hl,de

 add a,a
 dec a
 ld e,a
 ld a,b
 and %00000111
 inc a
 ld b,a

SDR8TR_NewRow:
 ld d,b
 ld a,(ix)
 inc ix

 rlca
SDR8TR_PrepMask:
 rrca
 djnz SDR8TR_PrepMask

 ld b,a
 and e
 cpl
 and (hl)
 ld (hl),a
 inc hl
 ld a,e
 cpl
 and b
 cpl
 and (hl)
 ld (hl),a
 dec hl

 ld b,d
 ex (sp),hl
 ld a,(hl)
 inc hl
 ex (sp),hl
 push bc

 rlca
SDR8TR_PrepByte:
 rrca
 djnz SDR8TR_PrepByte

 ld b,a
 and e
 or (hl)
 ld (hl),a
 inc hl
 ld a,e
 cpl
 and b
 or (hl)
 ld (hl),a

 ld bc,$10-1
 add hl,bc
 pop bc
 dec c
 jr nz,SDR8TR_NewRow
 pop hl
 ret